<?php
/**
 * Gravity Forms Peach Payments Gateway Add-On.
 *
 * @since     1.0
 * @package   GravityForms
 * @author    Peach Payments
 * @copyright Copyright (c) 2022, Peach Payments
 */

defined( 'ABSPATH' ) || die();

add_action('wp', array('GFPeach', 'maybe_thankyou_page'), 5);
// Include the payment add-on framework.
GFForms::include_payment_addon_framework();

class GFPeach extends GFPaymentAddOn {

	private static $_instance = null;

	protected $_version = GF_PEACHPAY_VER;
	protected $_min_gravityforms_version = '2.6.4';
	protected $_slug = 'gravityformspeachpayments';
	protected $_path = 'gravityformspeachpayments/peach.php';
	protected $_full_path = __FILE__;
	protected $_title = 'Gravity Forms Peach Payments Gateway Add-On';
	protected $_supports_callbacks = true;
	protected $_requires_credit_card = false;
	protected $_short_title = 'Peach Payments';
	protected $redirect_url = '';
	
	protected $_enable_rg_autoupgrade = true;
	protected $_requires_smallest_unit = true;
	protected $_capabilities_settings_page = 'gravityforms_peach';
	protected $_capabilities_form_settings = 'gravityforms_peach';
	protected $_capabilities_uninstall = 'gravityforms_peach_uninstall';
	protected $_capabilities = array( 'gravityforms_peach', 'gravityforms_peach_uninstall' );
	protected $_current_meta_key = '';
	protected $api = null;
	protected $_enable_rate_limits = true;
	protected $_has_settings_renderer;
	protected $_input_prefix = '';
	protected $_input_container_prefix = '';
	protected $billing_portal_handler;

	//Get an instance of this class.
	//DONE
	public static function get_instance() {

		if ( null === self::$_instance ) {
			self::$_instance = new GFPeach();
		}

		return self::$_instance;

	}
	
	public function pre_init() {
		
		parent::pre_init();

		//require_once 'includes/class-gf-field-peach-creditcard.php';
		
		$this->setup_peach_cron();
	}

	public function setup_peach_cron() {

		$cron_name = "{$this->_slug}_cron";
		$cron_name_payments = "{$this->_slug}_payments_cron";

		add_action( $cron_name, array( $this, 'peach_gravity_cron_funct' ) );
		add_action( $cron_name_payments, array( $this, 'peach_gravity_payments_cron_funct' ) );

		if ( ! wp_next_scheduled( $cron_name ) ) {
			wp_schedule_event( time(), 'hourly', $cron_name );
		}
		
		if ( ! wp_next_scheduled( $cron_name_payments ) ) {
			wp_schedule_event( time(), 'hourly', $cron_name_payments );
		}


	}
	
	public function plugin_settings_fields()
    {
        $description = '
			<p style="text-align: left;">' .
                       sprintf(
                           __(
                               'You will need a Peach Payments account in order to use the Peach Payments Add-On. Navigate to %sPeach Payments%s to register.',
                               'gravityformspeachpayments'
                           ),
                           '<a href="https://www.peachpayments.com/" target="_blank">',
                           '</a>'
                       ) .
                       '</p>
		    <p style="text-align: left;">The Peach Payments Feeds are configured per form.</p>
			<ul>
				<li>' . __(
                           'Navigate to \'Forms\' -> select \'Settings\' for the form, and select the \'Peach Payments\' tab.',
                           'gravityformspeachpayments'
                       ) . '</li>' .
                       '<li>' . __(
                           'From there, click \'Add New\' to configure Peach Payments feed settings for the currently selected form.',
                           'gravityformspeachpayments'
                       ) . '</li>' .
                       '</ul>
					   <p><b>'.__('API Settings','gravityformspeachpayments').':</b></p>';
					   
		$description_debug = '<p style="text-align: left;">' .
                       __(
                           'Enable \'Debug\' below to log the server-to-server communication between Peach Payments and your website, for each transaction. The log file for debugging can be found at /wp-content/plugins/gravityformspeachpayments/peach.log. If activated, be sure to protect it by adding an .htaccess file in the same directory. If not, the file will be readable by anyone. ',
                           'gravityformspeachpayments'
                       ) .
                       '</p>';
					   
		$fields = array(
			array(
				'title'       => esc_html__('How to configure Peach Payments', 'gravityformspeachpayments'),
                'description' => $description,
				'fields' => $this->api_settings_fields(),
			),
			array(
				'type'     => 'save',
				'messages' => array(
					'success' => __('Settings have been updated.', 'gravityformspeachpayments')
				),
			),
		);

        $fields[] = array(
			'title'       => esc_html__('Peach Payments Debug', 'gravityformspeachpayments'),
            'description' => $description_debug,
			'fields'      => array(
				array(
					'name'    => 'gf_peach_debug',
					'label'   => esc_html__('Enable/Disable', 'gravityformspeachpayments'),
					'type'    => 'checkbox',
					'choices' => array(
						array(
							'label' => __('', 'gravityformspeachpayments'),
							'name'  => 'gf_peach_debug'
						)
					)
				),
			),
		);
		
		return $fields;
		
    }
	
	public function api_settings_fields() {
		$webhook_fields = array(
			array(
				'name'       => 'transaction_mode_gf_peach',
				'label'       => __( 'Transaction Mode', 'gravityformspeachpayments' ),
				'type'    => 'select',
				//'multiple' => true,
				'choices' => array(
					array(
						'label' => 'Integrator Test',
						'value' => 'INTEGRATOR_TEST'
					),
					array(
						'label' => 'Connector Test',
						'value' => 'CONNECTOR_TEST'
					),
					array(
						'label' => 'Live',
						'value' => 'LIVE'
					),
				),
				'tooltip' => __( 'Set your gateway to LIVE when you are ready.', 'gravityformspeachpayments' )
			),
			array(
                'name'     => 'secret_gf_peach',
                'label'    => __('Secret Token', 'gravityformspeachpayments'),
                'type'     => 'text',
                'class'    => 'medium',
                'required' => true,
                'tooltip'  => __( 'This is the key generated within the Peach Payments Dashboard (Only if non-card payment method types have been enabled)', 'gravityformspeachpayments' )
            ),
            array(
                'name'     => 'channel_3ds_gf_peach',
                'label'    => __('3DSecure Channel ID', 'gravityformspeachpayments'),
                'type'     => 'text',
                'class'    => 'medium',
                'required' => true,
                'tooltip'  => __('The Entity ID that you received from Peach Payments.', 'gravityformspeachpayments')
            ),
		);
		
		return $webhook_fields;
	}
	
	public function can_create_feed() {
		return $this->is_peach_account_set();
	}
	
	private function is_peach_account_set(){
		//$token = $this->get_plugin_setting( 'access_token_gf_peach' );
		$secret = $this->get_plugin_setting( 'secret_gf_peach' );
		$channel = $this->get_plugin_setting( 'channel_3ds_gf_peach' );
		
		if($secret != '' && $channel != ''){
			return true;
		}
		
		return false;
	}
	
	public function feed_list_message() {
		if ( ! $this->can_create_feed() ) {
			return $this->requires_peach_api();
		}

		return GFFeedAddOn::feed_list_message();
	}
	
	public function requires_peach_api() {
		$url = '/wp-admin/admin.php?page=gf_settings&subview=gravityformspeachpayments';

		return sprintf( esc_html__( "You must add your Peach Payments API credentials before creating a feed. %sFinish Setup%s!", 'gravityformspeachpayments' ), "<a href='" . esc_url( $url ) . "'>", '</a>' );
	}
	
	public function get_menu_icon() {

		//return $this->is_gravityforms_supported( '2.5-beta-4' ) ? 'dashicons-money-alt' : 'dashicons';
		return file_get_contents( $this->get_base_path() . '/images/menu-icon.svg' );

	}
	
	public function feed_list_no_item_message()
    {
        $settings = $this->get_plugin_settings();
		$form = $this->get_current_form();
		
		$product = $email = $total = false;
		foreach($form['fields'] as $field){
			if($field->type == 'product'){
				$product = true;
			}
			if($field->type == 'email'){
				$email = true;
			}
			if($field->type == 'total'){
				$total = true;
			}
		}

		if($settings['secret_gf_peach'] == '' || $settings['channel_3ds_gf_peach'] == ''){
			$not_confirured = false;
		}else{
			$not_confirured = true;
		}
		
        if ( ! $not_confirured) {
            return sprintf(
                __('To get started, configure your %sPeach Settings%s!', 'gravityformsstripe'),
                '<a href="' . admin_url('admin.php?page=gf_settings&subview=' . $this->_slug) . '">',
                '</a>'
            );
        } else if(!$product){
			return __('To get started, you\'ll need at least one "Product" field added to your form.');
		} else if(!$email){
			return __('To get started, you\'ll need at least one "Email" field added to your form.');
		} else if(!$total){
			return __('To get started, you\'ll need at least one "Total" field added to your form.');
		} else {
            return parent::feed_list_no_item_message();
        }
    }
	
	public function feed_settings_fields()
    {
        $default_settings = parent::feed_settings_fields();
        //$default_settings = parent::add_field_after('feedName', $fields, $default_settings);
        $default_settings = $this->remove_field('recurringTimes', $default_settings);
        $default_settings = $this->remove_field('billingCycle', $default_settings);
		
		// Remove trial period
        $default_settings = $this->remove_field('trial', $default_settings);
		
		// Remove the subscription option from transaction type dropdown
		$transaction_type = $this->get_field( 'transactionType', $default_settings );
		foreach( $transaction_type['choices'] as $index => $choice ) {
			if( $choice['value'] == 'subscription' ) {
				unset( $transaction_type['choices'][$index] );
			}
		}
		$default_settings = $this->replace_field( 'transactionType', $transaction_type, $default_settings );

        //--add Page Style, Cancel URL
        $fields = array(
            array(
                'name'    => 'notifications',
                'label'   => __('Notifications', 'gravityformspeachpayments'),
                'type'    => 'notifications',
                'tooltip' => __('Notifications', 'gravityformspeachpayments') . __(
                        "Enable this option if you would like to only send out this form's notifications after payment has been received. Leaving this option disabled will send notifications immediately after the form is submitted.",
                        'gravityformspeachpayments'
                    )
            ),
        );

        //Add post fields if form has a post
        $form = $this->get_current_form();
        if (GFCommon::has_post_field($form['fields'])) {
            $post_settings = array(
                'name'    => 'post_checkboxes',
                'label'   => __('Posts', 'gravityformspeachpayments'),
                'type'    => 'checkbox',
                'tooltip' => __('Posts', 'gravityformspeachpayments') . __(
                        'Enable this option if you would like to only create the post after payment has been received.',
                        'gravityformspeachpayments'
                    ),
                'choices' => array(
                    array(
                        'label' => __('Create post only when payment is received.', 'gravityformspeachpayments'),
                        'name'  => 'delayPost'
                    ),
                ),
            );
            if ($this->get_setting('transactionType') == 'subscription') {
                $post_settings['choices'][] = array(
                    'label'    => __('Change post status when subscription is canceled.', 'gravityformspeachpayments'),
                    'name'     => 'change_post_status',
                    'onChange' => 'var action = this.checked ? "draft" : ""; jQuery("#update_post_action").val(action);',
                );
            }
            $fields[] = $post_settings;
        }
        
        $fields[]         = array(
            'name'  => 'custom_options',
            'label' => '',
            'type'  => 'custom',
        );
        $default_settings = $this->add_field_after('billingInformation', $fields, $default_settings);
        //-----------------------------------------------------------------------------------------

        //--get billing info section and add customer first/last name
        $billing_info   = parent::get_field('billingInformation', $default_settings);
        $billing_fields = $billing_info['field_map'];
        $add_first_name = true;
        $add_last_name  = true;
		$add_email  = true;
        foreach ($billing_fields as $mapping) {
            //add first/last name if it does not already exist in billing fields
            if ($mapping['name'] == 'firstName') {
                $add_first_name = false;
            } elseif ($mapping['name'] == 'lastName') {
                $add_last_name = false;
            } elseif ($mapping['name'] == 'email') {
                
            }
        }
        if ($add_last_name) {
            //add last name
            array_unshift(
                $billing_info['field_map'],
                array(
                    'name'     => 'lastName',
                    'label'    => __('Last Name', 'gravityformspeachpayments'),
                    'required' => true
                )
            );
        }
        if ($add_first_name) {
            array_unshift(
                $billing_info['field_map'],
                array(
                    'name'     => 'firstName',
                    'label'    => __('First Name', 'gravityformspeachpayments'),
                    'required' => true
                )
            );
        }
		
		$billing_info['field_map'][2]['required'] = 1;
		
		//$billing_info['field_map'][] = array( 'name' => 'recurringID', 'label' => esc_html__( 'Recurring ID', 'gravityformspeachpayments' ), 'required' => false );
		
        $default_settings = parent::replace_field('billingInformation', $billing_info, $default_settings);

        //----------------------------------------------------------------------------------------------------

        $default_settings = parent::remove_field('setupFee', $default_settings);
		$default_settings = parent::remove_field('conditionalLogic', $default_settings);

        //--Add Try to bill again after failed attempt.
        $freq             = array(
            'name'     => 'frequency',
            'label'    => __('Frequency', 'gravityformspeachpayments'),
            'type'     => 'select',
            //    'horizontal' => true,
            'required' => true,
            'choices'  => array(
				'minute'   => array( 'label' => esc_html__( 'minute(s)', 'gravityformspeachpayments' ), 'min' => 1, 'max' => 525600 ),
				'hour'   => array( 'label' => esc_html__( 'hour(s)', 'gravityformspeachpayments' ), 'min' => 1, 'max' => 8760 ),
                'day'   => array( 'label' => esc_html__( 'day(s)', 'gravityformspeachpayments' ), 'min' => 1, 'max' => 365 ),
				'week'  => array( 'label' => esc_html__( 'week(s)', 'gravityformspeachpayments' ), 'min' => 1, 'max' => 52 ),
				'month' => array( 'label' => esc_html__( 'month(s)', 'gravityformspeachpayments' ), 'min' => 1, 'max' => 12 ),
				'year'  => array( 'label' => esc_html__( 'year(s)', 'gravityformspeachpayments' ), 'min' => 1, 'max' => 10 )
            ),
            'tooltip'  => __('Frequency', 'gravityformspeachpayments') . __(
                    'Frequency.',
                    'gravityformspeachpayments'
                )
        );
        $initial          = array(
            'name'     => 'initialAmount',
            'label'    => esc_html__('Initial Amount', 'gravityforms'),
            'type'     => 'select',
            'choices'  => $this->recurring_amount_choices(),
            'required' => true,
            'tooltip'  => esc_html__('Initial Amount', 'gravityforms') . esc_html__(
                    "Select which field determines the initial payment amount, or select 'Form Total' to use the total of all pricing fields as the recurring amount.",
                    'gravityforms'
                ),
        );
		
        $default_settings = parent::add_field_after('recurringAmount', $initial, $default_settings);

        $default_settings = parent::add_field_after('initialAmount', $freq, $default_settings);


        $cycles           = array(
            'name'       => 'cycles',
            'label'      => __('Cycles (set to 0 for infinite)', 'gravityformspeachpayments'),
            'type'       => 'text',
            'horizontal' => true,
            'required'   => true,
            'tooltip'    => __('Cycles', 'gravityformspeachpayments') . __(
                    'Cycles',
                    'gravityformspeachpayments'
                )
        );
        $default_settings = parent::add_field_after('frequency', $cycles, $default_settings);


        //-----------------------------------------------------------------------------------------------------
        return apply_filters('gform_payfast_feed_settings_fields', $default_settings, $form);
    }
	
	public function redirect_url( $feed, $submission_data, $form, $entry ) {
		
		if ( ! rgempty('gf_peach_return', $_GET)) {
            return false;
        }
		
		GFAPI::update_entry_property($entry['id'], 'payment_status', 'Pending');
		
		$meta = rgar( $feed, 'meta' );
		$mode = $this->get_plugin_setting( 'transaction_mode_gf_peach' );
		if($mode == 'LIVE'){
			$url = 'https://secure.peachpayments.com/checkout/initiate';
			$ssl_verifypeer = true;
		}else{
			$url = 'https://testsecure.peachpayments.com/checkout/initiate';
			$ssl_verifypeer = false;
		}
		
		$currency = 'ZAR';
		if($entry['currency']){
			$currency = $entry['currency'];
		}
		$merchantTransactionId = 'PeachGFOrderNo-'.$form['id'].'-'.$entry['id'];
		$nonce = wp_create_nonce( $entry['id'].'_'.time() );
		$returnURL = $entry['source_url'];
		//$resultURL = $this->return_url($form['id'], $entry['id'], $entry['source_url']) . "&rm=2";
		$resultURL = $entry['source_url'];
		$siteURL = get_site_url().'/';
		
		//echo '<pre>'.print_r($resultURL, true).'</pre>'; die();
		$channel_3ds = $this->get_plugin_setting( 'channel_3ds_gf_peach' );
		
		$cancelUrl = $entry['source_url'];
		
		$data = array(
			"authentication.entityId" => $channel_3ds,
			"merchantTransactionId" => $merchantTransactionId,
			"amount" => $submission_data['payment_amount'], 
			"paymentType" => 'DB',
			"currency" => $currency, 
			"nonce" => $nonce,
			'originator' => 'gravityforms',
			"shopperResultUrl" => "".$resultURL."",
			"cancelUrl" => $cancelUrl 
			//"notificationUrl" => $entry['source_url']	
		);
		
		if($meta['transactionType'] == 'subscription'){
			$data['defaultPaymentMethod'] = 'CARD';
			$data['forceDefaultMethod'] = 'true';
			$data['createRegistration'] = 'true';
		}
				
		$data['customParameters[Gravity Forms]'] = $this->_version;
		
		if($meta['billingInformation_firstName'] != ''){
			$data['customer.givenName'] = $entry[$meta['billingInformation_firstName']];
		}
		if($meta['billingInformation_lastName'] != ''){
			$data ['customer.surname'] = $entry[$meta['billingInformation_lastName']];
		}
		if($submission_data['email'] != ''){
			$data['customer.email'] = $submission_data['email'];
		}
		
		//Check for valid address entry
		if($submission_data['address'] != '' && $submission_data['city'] != '' && $submission_data['state'] != '' && $submission_data['zip'] != '' && $submission_data['country'] != ''){
			$data['billing.street1'] = $submission_data['address'];
			if($submission_data['address2'] != ''){
				$data['billing.street2'] = $submission_data['address2'];
			}
			$data['billing.city'] = $submission_data['city'];
			$data['billing.state'] = $submission_data['state'];
			$data['billing.postcode'] = $submission_data['zip'];
			$data['billing.country'] = $submission_data['country'];
		}
		
		ksort($data);
		
		$sig_string = '';
		$dataFields = '';
		foreach($data as $key => $value){
			if(isset($key) && $value != ''){
				$sig_string .= $key.$value;
				$dataFields .= '&'.$key.'='.$value;
			}
		}
		
		$dataFields = ltrim($dataFields, '&');
		
		$secret = $this->get_plugin_setting( 'secret_gf_peach' );
		$signature = hash_hmac('sha256', $sig_string, $secret);
		
		$dataFields .= "&signature=" .$signature;
		
		$curl = curl_init();
		
		curl_setopt_array($curl, array(
		  CURLOPT_URL => $url,
		  CURLOPT_RETURNTRANSFER => true,
		  CURLOPT_ENCODING => '',
		  CURLOPT_MAXREDIRS => 10,
		  CURLOPT_TIMEOUT => 0,
		  CURLOPT_FOLLOWLOCATION => true,
		  CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
		  CURLOPT_CUSTOMREQUEST => 'POST',
		  CURLOPT_POSTFIELDS => $dataFields,
		  CURLOPT_HTTPHEADER => array(
			'Content-Type: application/x-www-form-urlencoded',
			'Accept: application/json',
			$siteURL
		  ),
		  ));
		  
		$responseData = curl_exec($curl);
		
		//echo '<pre>'.print_r(json_decode($responseData), true).'</pre>'; die();
		
		if(curl_errno($curl)) {
			$curlError = curl_error($curl);
			curl_close($curl);
			return false;
		}else{
			$response = json_decode($responseData);
			//echo '<pre>'.print_r($response, true).'</pre>'; die();
			
			if($response->redirectUrl){
				$returnURL = $response->redirectUrl;
			}else{
				curl_close($curl);
				return false;
			}
		}
		
		curl_close($curl);
		
		return $returnURL;
	}
	
	//Peach Cron
	//GF check_status()
	public static function peach_gravity_cron_funct(){
		global $wpdb;
		GFCommon::log_debug( __METHOD__ . '(): Checking Peach subscription status.' );

		// getting all peach subscription feeds
		//$recurring_feeds = $this->peach_feeds_by_slug( 'gravityformspeachpayments' );
		
		$sql = $wpdb->prepare( "SELECT * FROM {$wpdb->prefix}gf_addon_feed
                               WHERE addon_slug=%s {$form_filter} ORDER BY `feed_order`, `id` ASC", 'gravityformspeachpayments' );

		$recurring_feeds = $wpdb->get_results( $sql, ARRAY_A );
		foreach( $recurring_feeds as &$recurring_feed ) {
			$recurring_feed['meta'] = json_decode( $recurring_feed['meta'], true );
		}

		foreach ( $recurring_feeds as $feed ) {

			// process renewal's if peach feed is subscription feed
			if ( $feed['meta']['transactionType'] == 'subscription' ) {

				$form_id = $feed['form_id'];

				// getting billing cycle information
				$billing_cycle_number = $feed['meta']['cycles'];
				$billing_cycle_type   = $feed['meta']['frequency'];

				if ( $billing_cycle_type == 'minute(s)' ) {
					$billing_cycle = $billing_cycle_number . ' minutes';
				}else if ( $billing_cycle_type == 'hour(s)' ) {
					$billing_cycle = $billing_cycle_number . ' hours';
				}else if ( $billing_cycle_type == 'day(s)' ) {
					$billing_cycle = $billing_cycle_number . ' days';
				}else if ( $billing_cycle_type == 'week(s)' ) {
					$billing_cycle = $billing_cycle_number . ' weeks';
				}else if ( $billing_cycle_type == 'month(s)' ) {
					$billing_cycle = $billing_cycle_number . ' months';
				} else {
					$billing_cycle = $billing_cycle_number . ' years';
				}

				$querytime = strtotime( gmdate('Y-m-d H:i:s') . '-' . $billing_cycle );
				$nowtime = gmdate( 'Y-m-d H:i:s');
				$querydate = gmdate( 'Y-m-d H:i:s', $querytime );

				// Get entry table names and entry ID column.
				$entry_table      = self::get_entry_table_name();
				$entry_meta_table = self::get_entry_meta_table_name();
				$entry_id_column  = version_compare( self::get_gravityforms_db_version(), '2.3-dev-1', '<' ) ? 'lead_id' : 'entry_id';

				$results = $wpdb->get_results( "SELECT l.id, l.transaction_id, m.meta_value as payment_date
                                                FROM {$entry_table} l
                                                INNER JOIN {$entry_meta_table} m ON l.id = m.{$entry_id_column}
                                                WHERE l.form_id={$form_id}
                                                AND status = 'active'
												AND payment_status = 'Active'
                                                AND meta_key = 'subscription_payment_date'
                                                AND meta_value < '{$querydate}'" );
												
				$resultsJSON = json_encode($results);
				
				GFCommon::log_debug( __METHOD__ . '(): Count => ' .count($results) );

				foreach ( $results as $result ) {
					
					GFCommon::log_debug( __METHOD__ . '(): Lead with late payment' );

					$entry_id = $result->id;
					$entry    = GFAPI::get_entry( $entry_id );

					$subscription_id = $result->transaction_id;
					$payment_count = gform_get_meta( $entry_id, 'subscription_payment_count');
					
					GFCommon::log_debug( __METHOD__ . '(): ID => ' .$entry_id );
					GFCommon::log_debug( __METHOD__ . '(): Count => ' .$payment_count.' | '.$billing_cycle_number );
					
					//cancelled
					if($billing_cycle_number > $payment_count){
						$status = 'active';
					}else{
						$status = 'expired';
					}

					switch ( $status ) {
						case 'expired' :
						
							$apiClass = new GFPeach;
							$action = array(
								'subscription_id' => $subscription_id,
								'type'            => 'expire_subscription'
							);
							$finalResult = $apiClass->expire_subscription( $entry, $action );
							GFCommon::log_debug( __METHOD__ . '(): Expired => '. json_encode($finalResult) );

							break;

						default:
							GFCommon::log_debug( __METHOD__ . '(): Payment Action' );
							// getting feed trial information
							$trial_period_enabled     = $feed['meta']['trial_enabled'];
							$trial_period_occurrences = 1;

							// finding payment date
							$new_payment_time = strtotime( $result->payment_date . '+' . $billing_cycle );
							$new_payment_date = gmdate( 'Y-m-d H:i:s', $new_payment_time );

							// finding payment amount
							$payment_count      = gform_get_meta( $entry_id, 'subscription_payment_count' );
							$new_payment_amount = gform_get_meta( $entry_id, 'subscription_regular_amount' );
							if ( $trial_period_enabled == 1 && $trial_period_occurrences >= $payment_count ) {
								$new_payment_amount = gform_get_meta( $entry_id, 'subscription_trial_amount' );
							}

							gform_update_meta( $entry_id, 'subscription_payment_count', $payment_count + 1 );
							gform_update_meta( $entry_id, 'subscription_payment_date', $new_payment_date );
							
							$recurring_field = $feed['meta']['billingInformation_recurringID'];
							$id = rgar( $entry, $recurring_field );
							$currency = rgar( $entry, 'currency' );
							
							$apiClass = new GFPeach;
							$result = $apiClass->subscription_payment_api($id, $new_payment_amount, $feed['meta'], $currency);
							GFCommon::log_debug( __METHOD__ . '(): Result => '. json_encode($result) );
							$resultCode = $result->result->code;
							if($resultCode == '000.100.110'){
								$action = array(
									'amount'          => $new_payment_amount,
									'subscription_id' => $subscription_id,
									'type'            => 'add_subscription_payment'
								);
								$finalResult = $apiClass->add_subscription_payment( $entry, $action );
								GFCommon::log_debug( __METHOD__ . '(): Result => '. json_encode($finalResult) );
							}else{
								$note = sprintf( __( 'Subscription is currently suspended due to a transaction decline, rejection, or error. Suspended subscriptions must be reactivated before the next scheduled transaction or the subscription will be terminated by the payment gateway. Subscription Id: %s. Error: %s', 'gravityforms' ), $id,  print_r($result));
								$action = array(
									'note'            => $note,
									'subscription_id' => $subscription_id,
									'type'            => 'fail_subscription_payment'
								);
								$finalResult = $apiClass->fail_subscription_payment( $entry, $action );
								GFCommon::log_debug( __METHOD__ . '(): Failed Result => '. json_encode($finalResult) );
							}
							
							break;
					}


				}

			}

		}
	}
	
	public static function peach_gravity_payments_cron_funct(){
		global $wpdb;
		GFCommon::log_debug( __METHOD__ . '(): Checking Peach payment status.' );
		
		$sql = $wpdb->prepare( "SELECT * FROM {$wpdb->prefix}gf_addon_feed
                               WHERE addon_slug=%s {$form_filter} ORDER BY `feed_order`, `id` ASC", 'gravityformspeachpayments' );

		$recurring_feeds = $wpdb->get_results( $sql, ARRAY_A );
		foreach( $recurring_feeds as &$recurring_feed ) {
			$recurring_feed['meta'] = json_decode( $recurring_feed['meta'], true );
		}
		
		foreach ( $recurring_feeds as $feed ) {
		
			$results = $wpdb->get_results( "SELECT * FROM {$wpdb->prefix}gf_entry WHERE payment_status='Pending' AND status='active'" );									
			
			foreach($results as $result){
				
				$form = GFAPI::get_form($result->form_id);
				$lead = GFAPI::get_entry($result->id);
				$meta = rgar( $feed, 'meta' );
				
				$peach = GFPeach::get_instance();
				
				$mode = $peach->get_plugin_setting( 'transaction_mode_gf_peach' );
				if($mode == 'LIVE'){
					$url = 'https://secure.peachpayments.com/status';
					$ssl_verifypeer = true;
				}else{
					$url = 'https://testsecure.peachpayments.com/status';
					$ssl_verifypeer = false;
				}
				

				$authID = '';
				$channel_3ds = $peach->get_plugin_setting( 'channel_3ds_gf_peach' );
				$url .= '?authentication.entityId='.$channel_3ds;
				
				$merchantTransactionId = 'PeachGFOrderNo-'.$result->form_id.'-'.$result->id;
				$url .= '&merchantTransactionId='.$merchantTransactionId;
				
				$sig_string = 'authentication.entityId'.$channel_3ds.'merchantTransactionId'.$merchantTransactionId;
				$secret = $peach->get_plugin_setting( 'secret_gf_peach' );
				$signature = hash_hmac('sha256', $sig_string, $secret);
				$url .= '&signature='.$signature;
				
				$curl = curl_init();
				curl_setopt_array($curl, array(
				  CURLOPT_URL => $url,
				  CURLOPT_RETURNTRANSFER => true,
				  CURLOPT_ENCODING => '',
				  CURLOPT_MAXREDIRS => 10,
				  CURLOPT_TIMEOUT => 0,
				  CURLOPT_FOLLOWLOCATION => true,
				  CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
				  CURLOPT_CUSTOMREQUEST => 'GET',
				));
				
				$responseData = curl_exec($curl);
				$response = json_decode($responseData);
				
				GFCommon::log_debug( __METHOD__ . '(): Response => '. print_r($responseData, true));
				
				if(curl_errno($curl)) {
					$responseData = curl_error($curl);
					GFCommon::log_debug( __METHOD__ . '(): Pending Payment CURL Check ['.$result->id.'] => '. print_r($responseData, true));
				}else{
					//$resultCode = $response->result->code;
					$propresult = 'result.code';
					$resultCode = $response->$propresult;
					
					GFCommon::log_debug( __METHOD__ . '(): Check ResultCode ');
					GFCommon::log_debug( __METHOD__ . '(): Result Code => '. print_r($resultCode, true));
					
					if($resultCode == '000.100.110'){
						$peach = GFPeach::get_instance();
						
						$TransactionId = explode('-',$merchantTransactionId);
						
						GFAPI::update_entry_property($TransactionId[2], 'payment_status', 'Paid');
						GFAPI::update_entry_property($TransactionId[2], 'transaction_id', $merchantTransactionId);
						GFAPI::update_entry_property($TransactionId[2], 'payment_amount', $response->amount);
						gform_update_meta( $TransactionId[2], 'checkoutId', $response->checkoutId );
						
						if(isset($response->registrationId) && $response->registrationId != ''){
							$payType = 'create_subscription';
							$transactionType = '2';
							$subscription_field = $meta['billingInformation_recurringID'];
							GFAPI::update_entry_field( $TransactionId[2], $subscription_field, $response->registrationId );
							gform_update_meta( $TransactionId[2], 'subscription_payment_date', gmdate('Y-m-d H:i:s'));
							gform_update_meta( $TransactionId[2], 'subscription_payment_count', '1' );
							gform_update_meta( $TransactionId[2], 'subscription_regular_amount', $response->amount);
							GFAPI::update_entry_property($TransactionId[2], 'payment_status', 'Active');
							
						}else{
							$payType = 'complete_payment';
							$transactionType = '1';
						}
						
						GFAPI::update_entry_property($TransactionId[2], 'transaction_type', $transactionType);
						GFAPI::update_entry_property($TransactionId[2], 'payment_method', 'Peach Payments');
						GFAPI::update_entry_property($TransactionId[2], 'is_fulfilled', '1'); 
						GFAPI::update_entry_property($TransactionId[2], 'payment_date', gmdate('Y-m-d H:i:s'));
						
						$peach->insert_transaction($TransactionId[2], $payType, $merchantTransactionId, $response->amount);
						
						if (isset($meta['delayNotification']) && $meta['delayNotification'] != '') {
							//sending delayed notifications
							$notifications = $meta['selectedNotifications'];
							GFCommon::send_notifications($notifications, $form, $lead, true, 'form_submission');
						}
						
						/*
						GFFormDisplay::$submission[$TransactionId[1]] = array(
							'is_confirmation'      => true,
							'confirmation_message' => $confirmation,
							'form'                 => $form,
							'lead'                 => $lead
						);
						*/
						
						
					}else{
						GFCommon::log_debug( __METHOD__ . '(): Pending Payment Check ['.$result->id.'] => '. print_r($resultCode, true));
					}
				}
				curl_close($curl);
			}
		
		}
		
	}
	
	public function subscription_payment_api($id, $amount, $meta, $currency)
    {
		$mode = $this->get_plugin_setting( 'transaction_mode_gf_peach' );	
		if($mode == 'LIVE'){
			$url = 'https://eu-prod.oppwa.com/v1/registrations/'.$id.'/payments';
			$ssl_verifypeer = true;
		}else{
			$url = 'https://eu-test.oppwa.com/v1/registrations/'.$id.'/payments';
			$ssl_verifypeer = false;
		}
		
		$accesstoken = $this->get_plugin_setting( 'access_token_gf_peach' );
		$recurringid = $this->get_plugin_setting( 'channel_gf_peach' );
		
		$data = "entityId=" .$recurringid.
					"&amount=" .$amount.
					"&currency=" .$currency.
					"&paymentType=DB" .
					"&standingInstruction.mode=REPEATED" .
					"&standingInstruction.type=RECURRING" .
					"&standingInstruction.source=MIT";
	
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array(
					   'Authorization:Bearer '. $accesstoken));
		curl_setopt($ch, CURLOPT_POST, 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, $ssl_verifypeer);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_FAILONERROR, true);
		
		$responseData = curl_exec($ch);
		if(curl_errno($ch)) {
			$response = curl_error($ch);
		}else{
			$response = json_decode($responseData);
		}
		
		curl_close($ch);
		
		return $response;
	}
	
	public static function maybe_thankyou_page()
    {
		if(!empty($_POST) && isset($_POST['checkoutId'])){
			//[checkoutId] => 437cefb431084173840bf95e279e082d
			//[id] => 8ac7a4a185e6e1930185e845d2a14bb6
			//[signature] => c6d0b17ce35b8e5198681858328a0b19be38d982671f907eae08484538e87f3e
			//[timestamp] => 2023-01-25T09:32:53Z
			
			//[registrationId] => 8ac7a4a2860214c50186029f967c2cd3
			
			$peach = GFPeach::get_instance();
			
			$merchantTransactionId = $_POST['merchantTransactionId'];
			$TransactionId = explode('-',$merchantTransactionId);
			if($TransactionId[0] == 'PeachGFOrderNo'){
				
				//echo '<pre>'.print_r($_POST, true).'</pre>'; die();
				
				$form = GFAPI::get_form($TransactionId[1]);
				$lead = GFAPI::get_entry($TransactionId[2]);
				
				$feed = $peach->get_payment_feed($lead);
				$meta = rgar( $feed, 'meta' );
				
				if($_POST['result_code'] == '000.100.110'){
					
					GFAPI::update_entry_property($TransactionId[2], 'payment_status', 'Paid');
					GFAPI::update_entry_property($TransactionId[2], 'transaction_id', $merchantTransactionId);
					GFAPI::update_entry_property($TransactionId[2], 'payment_amount', $_POST['amount']);
					gform_update_meta( $TransactionId[2], 'checkoutId', $_POST['checkoutId'] );
					
					//echo '<pre>'.print_r($meta, true).'</pre>'; die();
					
					if(isset($_POST['registrationId']) && $_POST['registrationId'] != ''){
						$payType = 'create_subscription';
						$transactionType = '2';
						$subscription_field = $meta['billingInformation_recurringID'];
						GFAPI::update_entry_field( $TransactionId[2], $subscription_field, $_POST['registrationId'] );
						gform_update_meta( $TransactionId[2], 'subscription_payment_date', gmdate('Y-m-d H:i:s'));
						gform_update_meta( $TransactionId[2], 'subscription_payment_count', '1' );
						gform_update_meta( $TransactionId[2], 'subscription_regular_amount', $_POST['amount']);
						GFAPI::update_entry_property($TransactionId[2], 'payment_status', 'Active');
						//gform_update_meta( $TransactionId[2], 'subscription_trial_amount', $authorization['subscription']['subscription_trial_amount'] );
						
					}else{
						//$payType = 'payment';
						$payType = 'complete_payment';
						$transactionType = '1';
					}
					
					GFAPI::update_entry_property($TransactionId[2], 'transaction_type', $transactionType);
					GFAPI::update_entry_property($TransactionId[2], 'payment_method', 'Peach Payments');
					GFAPI::update_entry_property($TransactionId[2], 'is_fulfilled', '1'); 
					GFAPI::update_entry_property($TransactionId[2], 'payment_date', gmdate('Y-m-d H:i:s'));
					//echo '<pre>'.print_r($lead, true).'</pre>'; die();
					
					$peach->insert_transaction($TransactionId[2], $payType, $merchantTransactionId, $_POST['amount']);
					//process_itn($lead['id'], $payType, $merchantTransactionId, $_POST['amount']);
					
					if ( ! class_exists('GFFormDisplay')) {
						require_once(GFCommon::get_base_path() . '/form_display.php');
					}
					$confirmation = GFFormDisplay::handle_confirmation($form, $lead, false);
					if (is_array($confirmation) && isset($confirmation['redirect'])) {
						header("Location: {$confirmation['redirect']}");
						exit;
					}
					
					if (isset($meta['delayNotification']) && $meta['delayNotification'] != '') {
						//sending delayed notifications
						$notifications = $meta['selectedNotifications'];
						GFCommon::send_notifications($notifications, $form, $lead, true, 'form_submission');
					}
					
					GFFormDisplay::$submission[$TransactionId[1]] = array(
						'is_confirmation'      => true,
						'confirmation_message' => $confirmation,
						'form'                 => $form,
						'lead'                 => $lead
					);
				}else{
					
					$failed = array(
						'Form ID' => $TransactionId[1],
						'Entry' => $TransactionId[2],
						'Transaction ID' => $merchantTransactionId,
						'Code' => $_POST['result_code'],
						'Description' => $_POST['result_description'],
					);
					
					GFCommon::log_debug( __METHOD__ . '(): Failed Payment => '. json_encode($failed) );
					
					if(isset($_POST['registrationId']) && $_POST['registrationId'] != ''){
						$payType = 'create_subscription';
						$transactionType = '2';
						
					}else{
						$payType = 'complete_payment';
						$transactionType = '1';
					}
					
					GFAPI::update_entry_property($TransactionId[2], 'transaction_type', $transactionType);
					GFAPI::update_entry_property($TransactionId[2], 'payment_method', 'Peach Payments');
					GFAPI::update_entry_property($TransactionId[2], 'is_fulfilled', '0'); 
					GFAPI::update_entry_property($TransactionId[2], 'payment_date', gmdate('Y-m-d H:i:s'));
					
					$peach->insert_transaction($TransactionId[2], $payType, $merchantTransactionId, $_POST['amount']);
					
					if ( ! class_exists('GFFormDisplay')) {
						require_once(GFCommon::get_base_path() . '/form_display.php');
					}
					
					GFFormDisplay::$submission[$TransactionId[1]] = array(
						'is_confirmation'      => true,
						'confirmation_message' => 'Payment Failed. '.$_POST['result_description'].'.',
						'form'                 => $form,
						'lead'                 => $lead
					);
					
				}
			}
		}
	}
	
	public function process_itn($lead_id, $payType, $merchantTransactionId, $amount)
    {
		GFPaymentAddOn::insert_transaction($lead_id, $payType, $merchantTransactionId, $amount);
	}
	
	/*
	public function capture($authorization, $feed, $submission_data, $form, $entry){
		if(!empty($_POST) && isset($_POST['checkoutId'])){
			echo '<pre>'.print_r($_POST, true).'</pre>'; die();
		}
	}
	*/
	
	/*
	public function subscribe( $feed, $submission_data, $form, $entry ) {
		
	}
	*/
	

}